// +build !go1.5

package v4

import (
	"net/url"
	"strings"
)

func getURIPath(u *url.URL) string {
	var uri string

	if len(u.Opaque) > 0 {
		uri = "/" + strings.Join(strings.Split(u.Opaque, "/")[3:], "/")
	} else {
		uri = u.Path
	}

	if len(uri) == 0 {
		uri = "/"
	}

	return uri
}
