// +build !autogen

// Package dockerversion is auto-generated at build-time
package dockerversion

// Default build-time variable for library-import.
// This file is overridden on build with build-time informations.
const (
	GitCommit          string = "library-import"
	Version            string = "library-import"
	BuildTime          string = "library-import"
	IAmStatic          string = "library-import"
	ContainerdCommitID string = "library-import"
	RuncCommitID       string = "library-import"
	InitCommitID       string = "library-import"
)
